#!/usr/bin/env python
# -*- coding: utf-8 -*-

##
#  @file       image_blender.py
#  @brief      This script defines the ImageBlender class, which is used for blending two images together with adjustable blending ratio and gamma correction.
#

import cv2 as cv
import numpy as np
from gammaCorrector import GammaCorrector

##
#  @class      ImageBlender
#  @brief      A class for blending two images using a specified blend ratio and gamma correction.
#
class ImageBlender:
    # ˅

    # ˄

    ##
    #  @fn        __init__(self, gamma_corrector, blend_ratio=0.21)
    #  @brief     Constructor for ImageBlender.
    #  @param     gamma_corrector An instance of GammaCorrector for applying gamma correction.
    #  @param     blend_ratio The ratio at which images will be blended, default is 0.21.
    #
    def __init__(self, gamma_corrector, blend_ratio=0.21):
        # ˅
        self.gamma_corrector = gamma_corrector
        self.blend_ratio = blend_ratio
        # ˄

    ##
    #  @fn        blend_images(self, left_image, right_image)
    #  @brief     Blends two images together using the blend ratio and gamma correction.
    #  @param     left_image The left image to be blended.
    #  @param     right_image The right image to be blended.
    #  @return    A tuple containing the blended left and right images.
    #
    def blend_images(self, left_image, right_image):
        # ˅
        height, width, _ = left_image.shape
        blend_width = int(self.blend_ratio * width)

        alpha_left = np.linspace(1, 0, blend_width).reshape(1, blend_width, 1)
        alpha_left = np.repeat(alpha_left, height, axis=0)
        alpha_right = np.linspace(0, 1, blend_width).reshape(1, blend_width, 1)
        alpha_right = np.repeat(alpha_right, height, axis=0)

        left_overlap = left_image[:, -blend_width:]
        right_overlap = right_image[:, :blend_width]
        gamma_corrected_left_overlap = self.gamma_corrector.gamma_correction(left_overlap)
        gamma_corrected_right_overlap = self.gamma_corrector.gamma_correction(right_overlap)

        left_image[:, -blend_width:] = gamma_corrected_left_overlap
        right_image[:, :blend_width] = gamma_corrected_right_overlap

        left_image[:, -blend_width:] = (left_image[:, -blend_width:].astype(np.float32) * alpha_left).astype(np.uint8)
        right_image[:, :blend_width] = (right_image[:, :blend_width].astype(np.float32) * alpha_right).astype(np.uint8)

        return left_image, right_image
        # ˄

    ##
    #  @fn        process_images(self, left_image_path, right_image_path)
    #  @brief     Processes two images by applying gamma correction and blending them.
    #  @param     left_image_path Path to the left image.
    #  @param     right_image_path Path to the right image.
    #  @return    A tuple of adjusted left, adjusted right, and blended images.
    #  @exception FileNotFoundError If one of the input images is not found.
    #
    def process_images(self, left_image_path, right_image_path):
        # ˅
        left_image = cv.imread(left_image_path, cv.IMREAD_COLOR)
        right_image = cv.imread(right_image_path, cv.IMREAD_COLOR)

        if left_image is None or right_image is None:
            raise FileNotFoundError("One of the input images is not found.")

        gamma_corrected_left = self.gamma_corrector.gamma_correction(left_image)
        gamma_corrected_right = self.gamma_corrector.gamma_correction(right_image)

        adjusted_left, adjusted_right = self.blend_images(gamma_corrected_left, gamma_corrected_right)

        height, width, _ = left_image.shape
        blend_width = int(self.blend_ratio * width)

        left_non_overlap = adjusted_left[:, :-blend_width]
        right_non_overlap = adjusted_right[:, blend_width:]

        left_blend = adjusted_left[:, -blend_width:]
        right_blend = adjusted_right[:, :blend_width]
        blended_area = left_blend + right_blend

        blended_image = np.hstack((left_non_overlap, blended_area, right_non_overlap))

        return adjusted_left, adjusted_right, blended_image
        # ˄

    # ˅

    # ˄
